@@ -11,9 +11,10 @@ import logging
 import os
 import pickle
 from abc import ABCMeta, abstractmethod
-from collections import OrderedDict
+from typing import Dict, List, overload, Union
 from uuid import uuid4
 
+from typing_extensions import Literal, TypedDict
 
 from galaxy.exceptions import MessageException
 from galaxy.util import (
@@ -74,14 +75,33 @@ SUPPORTED_WORKFLOW_REQUIREMENTS = SUPPORTED_TOOL_REQUIREMENTS + [
 
 PERSISTED_REPRESENTATION = "cwl_tool_object"
 
+ToolStateType = Dict[str, Union[None, str, bool, Dict[str, str]]]
+InputInstanceDict = TypedDict(
+    "InputInstanceDict",
+    {
+        "type": str,
+        "name": str,
+        "title": str,
+        "label": str,
+        "help": str,
+        "optional": bool,
+        "area": bool,
+        "value": str,
+        "collection_type": str,
+    },
+    total=False,
+)
+InputInstanceArrayDict = TypedDict(
+    "InputInstanceArrayDict",
+    {"type": str, "name": str, "title": str, "blocks": List[InputInstanceDict]},
+)
+
 
 def tool_proxy(tool_path=None, tool_object=None, strict_cwl_validation=True, tool_directory=None, uuid=None):
     """ Provide a proxy object to cwltool data structures to just
     grab relevant data.
     """
     ensure_cwltool_available()
-    # if uuid is None:
-    #    raise Exception("tool_proxy must be called with non-None uuid")
     tool = _to_cwl_tool_object(
         tool_path=tool_path,
         tool_object=tool_object,
@@ -151,7 +171,7 @@ def _to_cwl_tool_object(tool_path=None, tool_object=None, cwl_tool_object=None,
         path = tool_directory
         if path is None:
             path = os.getcwd()
-        uri = ref_resolver.file_uri(path) + "/"
+        uri = f"{ref_resolver.file_uri(path)}/"
         sourceline.add_lc_filename(tool_object, uri)
         raw_process_reference = schema_loader.raw_process_reference_for_object(
             tool_object,
@@ -210,7 +230,7 @@ def _schema_loader(strict_cwl_validation):
 
 
 def _hack_cwl_requirements(cwl_tool):
-    move_to_hints = []
+    move_to_hints: List[int] = []
     for i, requirement in enumerate(cwl_tool.requirements):
         if requirement["class"] == DOCKER_REQUIREMENT:
             move_to_hints.insert(0, i)
@@ -229,7 +249,7 @@ def check_requirements(rec, tool=True):
                 else:
                     possible = SUPPORTED_WORKFLOW_REQUIREMENTS
                 if r["class"] not in possible:
-                    raise Exception("Unsupported requirement %s" % r["class"])
+                    raise Exception(f"Unsupported requirement {r['class']}")
         for d in rec:
             check_requirements(rec[d], tool=tool)
     if isinstance(rec, list):
@@ -239,11 +259,18 @@ def check_requirements(rec, tool=True):
 
 class ToolProxy(metaclass=ABCMeta):
 
+    _class: str
+
     def __init__(self, tool, uuid, raw_process_reference=None, tool_path=None):
         self._tool = tool
-        self._uuid = uuid
+        self.uuid = uuid
         self._tool_path = tool_path
         self._raw_process_reference = raw_process_reference
+        # remove input parameter formats from CWL files so that cwltool
+        # does not complain they are missing in the input data
+        for input_field in self._tool.inputs_record_schema["fields"]:
+            if 'format' in input_field:
+                del input_field['format']
 
     def job_proxy(self, input_dict, output_dict, job_directory="."):
         """ Build a cwltool.job.Job describing computation using a input_json
@@ -257,7 +284,7 @@ class ToolProxy(metaclass=ABCMeta):
         raw_id = self._tool.tool.get("id", None)
         return raw_id
 
-    def galaxy_id(self):
+    def galaxy_id(self) -> str:
         raw_id = self.id
         tool_id = None
         # don't reduce "search.cwl#index" to search
@@ -265,8 +292,7 @@ class ToolProxy(metaclass=ABCMeta):
             tool_id = os.path.basename(raw_id)
             # tool_id = os.path.splitext(os.path.basename(raw_id))[0]
         if not tool_id:
-            return self._uuid
-        assert tool_id
+            return str(self.uuid)
         if tool_id.startswith("#"):
             tool_id = tool_id[1:]
         return tool_id
@@ -303,7 +329,7 @@ class ToolProxy(metaclass=ABCMeta):
         return {
             "class": self._class,
             "pickle": unicodify(base64.b64encode(pickle.dumps(persisted_obj, pickle.HIGHEST_PROTOCOL))),
-            "uuid": self._uuid,
+            "uuid": self.uuid,
         }
 
     @staticmethod
@@ -459,7 +485,7 @@ class JobProxy:
             )
 
             args = []
-            kwargs = {}
+            kwargs: Dict[str, str] = {}
             if RuntimeContext is not None:
                 args.append(RuntimeContext(job_args))
             else:
@@ -509,7 +535,7 @@ class JobProxy:
             def stage_recursive(value):
                 is_list = isinstance(value, list)
                 is_dict = isinstance(value, dict)
-                log.info("handling value {}, is_list {}, is_dict {}".format(value, is_list, is_dict))
+                log.info(f"handling value {value}, is_list {is_list}, is_dict {is_dict}")
                 if is_list:
                     for val in value:
                         stage_recursive(val)
@@ -522,7 +548,7 @@ class JobProxy:
                             if not os.path.exists(staged_loc):
                                 os.symlink(location, staged_loc)
                             value["location"] = staged_loc
-                    for key, dict_value in value.items():
+                    for dict_value in value.values():
                         stage_recursive(dict_value)
                 else:
                     log.info("skipping simple value...")
@@ -582,7 +608,7 @@ class JobProxy:
         else:
             self._ok = False
 
-        log.info("Output are {}, status is {}".format(out, process_status))
+        log.info(f"Output are {out}, status is {process_status}")
 
     def collect_outputs(self, tool_working_directory, rcode):
         if not self.is_command_line_job:
@@ -594,7 +620,7 @@ class JobProxy:
             else:
                 cwl_job.run()
             if not self._ok:
-                raise Exception("Final process state not ok, [%s]" % self._process_status)
+                raise Exception(f"Final process state not ok, [{self._process_status}]")
             return self._final_output
         else:
             return self.cwl_job().collect_outputs(tool_working_directory, rcode)
@@ -611,7 +637,7 @@ class JobProxy:
 
     def _output_extra_files_dir(self, output_name):
         output_id = self.output_id(output_name)
-        return os.path.join(self._job_directory, "dataset_%s_files" % output_id)
+        return os.path.join(self._job_directory, "outputs", f"dataset_{output_id}_files")
 
     def output_id(self, output_name):
         output_id = self._output_dict[output_name]["id"]
@@ -636,7 +662,7 @@ class JobProxy:
         cwl_job = self.cwl_job()
 
         def stageFunc(resolved_path, target_path):
-            log.info("resolving {} to {}".format(resolved_path, target_path))
+            log.info(f"resolving {resolved_path} to {target_path}")
             try:
                 os.symlink(resolved_path, target_path)
             except OSError:
@@ -651,7 +677,7 @@ class JobProxy:
             generate_mapper = pathmapper.PathMapper(cwl_job.generatefiles["listing"],
                                                     outdir, outdir, separateDirs=False)
             # TODO: figure out what inplace_update should be.
-            inplace_update = getattr(cwl_job, "inplace_update")
+            inplace_update = cwl_job.inplace_update
             process.stage_files(generate_mapper, stageFunc, ignore_writable=inplace_update, symlink=False)
             relink_initialworkdir(generate_mapper, outdir, outdir, inplace_update=inplace_update)
         # else: expression tools do not have a path mapper.
@@ -720,7 +746,7 @@ class WorkflowProxy:
     def cwl_ids_to_index(self, step_proxies):
         index = 0
         cwl_ids_to_index = {}
-        for i, input_dict in enumerate(self._workflow.tool['inputs']):
+        for input_dict in self._workflow.tool['inputs']:
             cwl_ids_to_index[input_dict["id"]] = index
             index += 1
 
@@ -738,7 +764,7 @@ class WorkflowProxy:
         cwl_ids_to_index = self.cwl_ids_to_index(step_proxies)
         input_connections_by_step = []
         for step_proxy in step_proxies:
-            input_connections_step = {}
+            input_connections_step: Dict[str, List[Dict[str, str]]] = {}
             for input_proxy in step_proxy.input_proxies:
                 cwl_source_id = input_proxy.cwl_source_id
                 input_name = input_proxy.input_name
@@ -795,7 +821,7 @@ class WorkflowProxy:
             if self.jsonld_id_to_label(input["id"]) == label:
                 return i
 
-        raise Exception("Failed to find index for label %s" % label)
+        raise Exception(f"Failed to find index for label {label}")
 
     def jsonld_id_to_label(self, id):
         if "#" in self.cwl_id:
@@ -828,15 +854,17 @@ class WorkflowProxy:
                 input_as_dict["type"] = "parameter_input"
                 # TODO: dispatch on actual type so this doesn't always need
                 # to be field - simpler types could be simpler inputs.
-                tool_state = {}
+                tool_state: ToolStateType = {}
                 tool_state["parameter_type"] = "field"
+                default_set = "default" in input
                 default_value = input.get("default")
-                optional = False
+                optional = default_set
                 if isinstance(input_type, list) and "null" in input_type:
                     optional = True
                 if not optional and isinstance(input_type, dict) and "type" in input_type:
-                    assert False
-                tool_state["default_value"] = {"src": "json", "value": default_value}
+                    raise ValueError("'type' detected in non-optional input dictionary.")
+                if default_set:
+                    tool_state["default"] = {"src": "json", "value": default_value}
                 tool_state["optional"] = optional
                 input_as_dict["tool_state"] = tool_state
             else:
@@ -868,7 +896,7 @@ def split_step_references(step_references, workflow_id=None, multiple=True):
                 sep_on = "#"
             expected_prefix = workflow_id + sep_on
             if not step_reference.startswith(expected_prefix):
-                raise AssertionError("step_reference [{}] doesn't start with {}".format(step_reference, expected_prefix))
+                raise AssertionError(f"step_reference [{step_reference}] doesn't start with {expected_prefix}")
             step_reference = step_reference[len(expected_prefix):]
 
         # Now just grab the step name and input/output name.
@@ -968,7 +996,7 @@ class InputProxy:
         cwl_source_id = cwl_input.get("source", None)
         if cwl_source_id is None:
             if "valueFrom" not in cwl_input and "default" not in cwl_input:
-                msg = "Workflow step input must define a source, a valueFrom, or a default value. Obtained [%s]." % cwl_input
+                msg = f"Workflow step input must define a source, a valueFrom, or a default value. Obtained [{cwl_input}]."
                 raise MessageException(msg)
 
         assert cwl_input_id
@@ -1015,7 +1043,7 @@ class ToolStepProxy(BaseStepProxy):
 
     @property
     def tool_proxy(self):
-        # Neeeds to be cached so UUID that is loaded matches UUID generated with to_dict.
+        # Needs to be cached so UUID that is loaded matches UUID generated with to_dict.
         if self._tool_proxy is None:
             self._tool_proxy = _cwl_tool_object_to_proxy(self.cwl_tool_object, uuid=str(uuid4()))
         return self._tool_proxy
@@ -1028,14 +1056,14 @@ class ToolStepProxy(BaseStepProxy):
         # We need to stub out null entries for things getting replaced by
         # connections. This doesn't seem ideal - consider just making Galaxy
         # handle this.
-        tool_state = {}
+        tool_state: ToolStateType = {}
         for input_name in input_connections.keys():
             tool_state[input_name] = None
 
         outputs = self.galaxy_workflow_outputs_list()
         return {
             "id": self._index,
-            "tool_uuid": self.tool_proxy._uuid,  # TODO: make sure this is respected...
+            "tool_uuid": self.tool_proxy.uuid,  # TODO: make sure this is respected...
             "label": self.label,
             "position": {"left": 0, "top": 0},
             "type": "tool",
@@ -1114,7 +1142,7 @@ def _outer_field_to_input_instance(field):
     name, label, description = _field_metadata(field)
 
     case_name = "_cwl__type_"
-    case_label = "Specify Parameter %s As" % label
+    case_label = f"Specify Parameter {label} As"
 
     def value_input(type_description):
         value_name = "_cwl__value_"
@@ -1134,7 +1162,7 @@ def _outer_field_to_input_instance(field):
     for type_description in type_descriptions:
         select_options.append({"value": type_description.name, "label": type_description.label})
         input_instances = []
-        if type_description.uses_param():
+        if type_description.uses_param:
             input_instances.append(value_input(type_description))
         case_options.append((type_description.name, input_instances))
 
@@ -1209,7 +1237,7 @@ class ConditionalInstance:
             name=self.name,
             type=INPUT_TYPE.CONDITIONAL,
             test=self.case.to_dict(),
-            when=OrderedDict(),
+            when={},
         )
         for value, block in self.whens:
             as_dict["when"][value] = [i.to_dict() for i in block]
@@ -1250,18 +1278,30 @@ class InputInstance:
         self.array = array
         self.area = area
 
-    def to_dict(self, itemwise=True):
+    @overload
+    def to_dict(self, itemwise: Literal[False]) -> InputInstanceDict:
+        ...
+
+    @overload
+    def to_dict(
+        self, itemwise: Literal[True]
+    ) -> Union[InputInstanceDict, InputInstanceArrayDict]:
+        ...
+
+    def to_dict(
+        self, itemwise: bool = True
+    ) -> Union[InputInstanceDict, InputInstanceArrayDict]:
         if itemwise and self.array:
-            as_dict = dict(
+            return InputInstanceArrayDict(
                 type="repeat",
-                name="%s_repeat" % self.name,
-                title="%s" % self.name,
+                name=f"{self.name}_repeat",
+                title=f"{self.name}",
                 blocks=[
                     self.to_dict(itemwise=False)
                 ]
             )
         else:
-            as_dict = dict(
+            as_dict = InputInstanceDict(
                 name=self.name,
                 label=self.label or self.name,
                 help=self.description,
@@ -1277,8 +1317,7 @@ class InputInstance:
                 as_dict["value"] = "0.0"
             elif self.input_type == INPUT_TYPE.DATA_COLLECTON:
                 as_dict["collection_type"] = self.collection_type
-
-        return as_dict
+            return as_dict
 
 
 OUTPUT_TYPE = Bunch(
