@@ -4,12 +4,12 @@ import json
 import logging
 import os
 import re
+import shlex
 import shutil
 import sys
 import tempfile
 
 import packaging.version
-from six.moves import shlex_quote
 
 from galaxy.util import (
     commands,
@@ -26,12 +26,10 @@ SHELL_UNSAFE_PATTERN = re.compile(r"[\s\"']")
 
 IS_OS_X = sys.platform == "darwin"
 
-# BSD 3-clause
-CONDA_LICENSE = "http://docs.continuum.io/anaconda/eula"
 VERSIONED_ENV_DIR_NAME = re.compile(r"__(.*)@(.*)")
 UNVERSIONED_ENV_DIR_NAME = re.compile(r"__(.*)@_uv_")
 USE_PATH_EXEC_DEFAULT = False
-CONDA_VERSION = "4.6.14"
+CONDA_PACKAGE_SPECS = ("conda=4.6.14", "'pyopenssl>=22.1.0'")
 CONDA_BUILD_VERSION = "3.17.8"
 USE_LOCAL_DEFAULT = False
 
@@ -105,7 +103,6 @@ class CondaContext(installable.InstallableContext):
             ensure_channels = None
         self.ensure_channels = ensure_channels
         self._conda_version = None
-        self._miniconda_version = None
         self._conda_build_available = None
         self.use_local = use_local
 
@@ -122,35 +119,16 @@ class CondaContext(installable.InstallableContext):
         return self._conda_build_available
 
     def _guess_conda_properties(self):
-        conda_meta_path = self._conda_meta_path
-        # Perhaps we should call "conda info --json" and parse it but for now we are going
-        # to assume the default.
-        conda_version = packaging.version.parse(CONDA_VERSION)
-        conda_build_available = False
-        miniconda_version = "3"
-
-        if os.path.exists(conda_meta_path):
-            for package in os.listdir(conda_meta_path):
-                package_parts = package.split("-")
-                if len(package_parts) < 3:
-                    continue
-                package = '-'.join(package_parts[:-2])
-                version = package_parts[-2]
-                # build = package_parts[-1]
-                if package == "conda":
-                    conda_version = packaging.version.parse(version)
-                if package == "python" and version.startswith("2"):
-                    miniconda_version = "2"
-                if package == "conda-build":
-                    conda_build_available = True
-
-        self._conda_version = conda_version
-        self._miniconda_version = miniconda_version
-        self._conda_build_available = conda_build_available
-
-    @property
-    def _conda_meta_path(self):
-        return os.path.join(self.conda_prefix, "conda-meta")
+        info = self.conda_info()
+        self._conda_version = packaging.version.parse(info["conda_version"])
+        self._conda_build_available = False
+        conda_build_version = info.get("conda_build_version")
+        if conda_build_version != "not installed":
+            try:
+                self._conda_version = packaging.version.parse(conda_build_version)
+                self._conda_build_available = True
+            except Exception:
+                pass
 
     @property
     def _override_channels_args(self):
@@ -226,12 +204,12 @@ class CondaContext(installable.InstallableContext):
         env = {}
         if self.condarc_override:
             env["CONDARC"] = self.condarc_override
-        cmd_string = ' '.join(map(shlex_quote, cmd))
+        cmd_string = ' '.join(map(shlex.quote, cmd))
         kwds = dict()
         try:
             if stdout_path:
                 kwds['stdout'] = open(stdout_path, 'w')
-                cmd_string += " > '%s'" % stdout_path
+                cmd_string += f" > '{stdout_path}'"
             conda_exec_home = env['HOME'] = tempfile.mkdtemp(prefix='conda_exec_home_')  # We don't want to pollute ~/.conda, which may not even be writable
             log.debug("Executing command: %s", cmd_string)
             return self.shell_exec(cmd, env=env, **kwds)
@@ -248,10 +226,9 @@ class CondaContext(installable.InstallableContext):
         """
         Return the process exit code (i.e. 0 in case of success).
         """
-        create_args = [
-            "-y",
-            "--quiet"
-        ]
+        create_args = ["-y", "--quiet"]
+        if self.conda_version >= packaging.version.parse("4.7.5"):
+            create_args.append("--strict-channel-priority")
         if allow_local and self.use_local:
             create_args.extend(["--use-local"])
         create_args.extend(self._override_channels_args)
@@ -275,9 +252,9 @@ class CondaContext(installable.InstallableContext):
         """
         Return the process exit code (i.e. 0 in case of success).
         """
-        install_args = [
-            "-y"
-        ]
+        install_args = ["-y"]
+        if self.conda_version >= packaging.version.parse("4.7.5"):
+            install_args.append("--strict-channel-priority")
         if allow_local and self.use_local:
             install_args.append("--use-local")
         install_args.extend(self._override_channels_args)
@@ -360,26 +337,26 @@ class CondaTarget:
 
     def __init__(self, package, version=None, channel=None):
         if SHELL_UNSAFE_PATTERN.search(package) is not None:
-            raise ValueError("Invalid package [%s] encountered." % package)
+            raise ValueError(f"Invalid package [{package}] encountered.")
         self.package = package
         if version and SHELL_UNSAFE_PATTERN.search(version) is not None:
-            raise ValueError("Invalid version [%s] encountered." % version)
+            raise ValueError(f"Invalid version [{version}] encountered.")
         self.version = version
         if channel and SHELL_UNSAFE_PATTERN.search(channel) is not None:
-            raise ValueError("Invalid version [%s] encountered." % channel)
+            raise ValueError(f"Invalid version [{channel}] encountered.")
         self.channel = channel
 
     def __str__(self):
-        attributes = "package=%s" % self.package
+        attributes = f"package={self.package}"
         if self.version is not None:
-            attributes = "{},version={}".format(self.package, self.version)
+            attributes = f"{self.package},version={self.version}"
         else:
-            attributes = "%s,unversioned" % self.package
+            attributes = f"{self.package},unversioned"
 
         if self.channel:
             attributes = "%s,channel=%s" % self.channel
 
-        return "CondaTarget[%s]" % attributes
+        return f"CondaTarget[{attributes}]"
 
     __repr__ = __str__
 
@@ -388,7 +365,7 @@ class CondaTarget:
         """ Return a package specifier as consumed by conda install/create.
         """
         if self.version:
-            return "{}={}".format(self.package, self.version)
+            return f"{self.package}={self.version}"
         else:
             return self.package
 
@@ -399,9 +376,9 @@ class CondaTarget:
         a fixed and predictable name given package and version.
         """
         if self.version:
-            return "__{}@{}".format(self.package, self.version)
+            return f"__{self.package}@{self.version}"
         else:
-            return "__%s@_uv_" % (self.package)
+            return f"__{self.package}@_uv_"
 
     def __hash__(self):
         return hash((self.package, self.version, self.channel))
@@ -428,15 +405,13 @@ def hash_conda_packages(conda_packages, conda_target=None):
 # shell makes sense for planemo, in Galaxy this should just execute
 # these commands as Python
 def install_conda(conda_context, force_conda_build=False):
-    f, script_path = tempfile.mkstemp(suffix=".sh", prefix="conda_install")
-    os.close(f)
+    with tempfile.NamedTemporaryFile(suffix=".sh", prefix="conda_install", delete=False) as temp:
+        script_path = temp.name
     download_cmd = commands.download_command(conda_link(), to=script_path)
     install_cmd = ['bash', script_path, '-b', '-p', conda_context.conda_prefix]
-    package_targets = [
-        "conda=%s" % CONDA_VERSION,
-    ]
+    package_targets = list(CONDA_PACKAGE_SPECS)
     if force_conda_build or conda_context.use_local:
-        package_targets.append("conda-build=%s" % CONDA_BUILD_VERSION)
+        package_targets.append(f"conda-build={CONDA_BUILD_VERSION}")
     log.info("Installing conda, this may take several minutes.")
     try:
         exit_code = conda_context.shell_exec(download_cmd)
