@@ -26,6 +26,7 @@ log = getLogger(__name__)
 
 DOCKER_CONTAINER_TYPE = "docker"
 SINGULARITY_CONTAINER_TYPE = "singularity"
+TRAP_KILL_CONTAINER = "trap _on_exit EXIT"
 
 LOAD_CACHED_IMAGE_COMMAND_TEMPLATE = r'''
 python << EOF
@@ -117,6 +118,10 @@ def preprocess_volumes(volumes_raw_str, container_type):
     ro for Singularity iff no subdirectories are rw (Singularity does not allow ro
     parent directories with rw subdirectories).
 
+    >>> preprocess_volumes(None, DOCKER_CONTAINER_TYPE)
+    []
+    >>> preprocess_volumes("", DOCKER_CONTAINER_TYPE)
+    []
     >>> preprocess_volumes("/a/b", DOCKER_CONTAINER_TYPE)
     ['/a/b:rw']
     >>> preprocess_volumes("/a/b:ro,/a/b/c:rw", DOCKER_CONTAINER_TYPE)
@@ -131,6 +136,9 @@ def preprocess_volumes(volumes_raw_str, container_type):
     ['/a/b', '/a/b/c']
     """
 
+    if not volumes_raw_str:
+        return []
+
     volumes_raw_strs = [v.strip() for v in volumes_raw_str.split(",")]
     volumes = []
     rw_paths = []
@@ -138,11 +146,11 @@ def preprocess_volumes(volumes_raw_str, container_type):
     for volume_raw_str in volumes_raw_strs:
         volume_parts = volume_raw_str.split(":")
         if len(volume_parts) > 3:
-            raise Exception("Unparsable volumes string in configuration [%s]" % volumes_raw_str)
+            raise Exception(f"Unparsable volumes string in configuration [{volumes_raw_str}]")
         if len(volume_parts) == 3:
-            volume_parts = ["{}:{}".format(volume_parts[0], volume_parts[1]), volume_parts[2]]
+            volume_parts = [f"{volume_parts[0]}:{volume_parts[1]}", volume_parts[2]]
         if len(volume_parts) == 2 and volume_parts[1] not in ("rw", "ro", "default_ro"):
-            volume_parts = ["{}:{}".format(volume_parts[0], volume_parts[1]), "rw"]
+            volume_parts = [f"{volume_parts[0]}:{volume_parts[1]}", "rw"]
         if len(volume_parts) == 1:
             volume_parts.append("rw")
         volumes.append(volume_parts)
@@ -294,7 +302,7 @@ class DockerContainer(Container, HasDockerLikeVolumes):
 
         working_directory = self.job_info.working_directory
         if not working_directory:
-            raise Exception("Cannot containerize command [%s] without defined working directory." % working_directory)
+            raise Exception(f"Cannot containerize command [{working_directory}] without defined working directory.")
 
         volumes_raw = self._expand_volume_str(self.destination_info.get("docker_volumes", "$defaults"))
         preprocessed_volumes_list = preprocess_volumes(volumes_raw, self.container_type)
@@ -332,12 +340,13 @@ class DockerContainer(Container, HasDockerLikeVolumes):
         # https://stackoverflow.com/questions/34228864/stop-and-delete-docker-container-if-its-running
         # Standard error is:
         #    Error response from daemon: Cannot kill container: 2b0b961527574ebc873256b481bbe72e: No such container: 2b0b961527574ebc873256b481bbe72e
-        return """
+        return f"""
 _on_exit() {{
-  {} &> /dev/null
+  {kill_command} &> /dev/null
 }}
-trap _on_exit 0
-{}\n{}""".format(kill_command, cache_command, run_command)
+{TRAP_KILL_CONTAINER}
+{cache_command}
+{run_command}"""
 
     def __cache_from_file_command(self, cached_image_file, docker_host_props):
         images_cmd = docker_util.build_docker_images_command(truncate=False, **docker_host_props)
@@ -356,7 +365,7 @@ trap _on_exit 0
         return cache_path if os.path.exists(cache_path) else None
 
     def __get_destination_overridable_property(self, name):
-        prop_name = "docker_%s" % name
+        prop_name = f"docker_{name}"
         if prop_name in self.destination_info:
             return self.destination_info[prop_name]
         else:
@@ -365,7 +374,7 @@ trap _on_exit 0
 
 def docker_cache_path(cache_directory, container_id):
     file_container_id = container_id.replace("/", "_slash_")
-    cache_file_name = "docker_%s.tar" % file_container_id
+    cache_file_name = f"docker_{file_container_id}.tar"
     return os.path.join(cache_directory, cache_file_name)
 
 
@@ -392,11 +401,18 @@ class SingularityContainer(Container, HasDockerLikeVolumes):
             **self.get_singularity_target_kwds()
         )
 
+    def build_singularity_pull_command(self, cache_path):
+        return singularity_util.pull_singularity_command(
+            image_identifier=self.container_id,
+            cache_path=cache_path,
+            **self.get_singularity_target_kwds()
+        )
+
     def containerize_command(self, command):
 
         env = []
         for pass_through_var in self.tool_info.env_pass_through:
-            env.append((pass_through_var, "$%s" % pass_through_var))
+            env.append((pass_through_var, f"${pass_through_var}"))
 
         # Allow destinations to explicitly set environment variables just for
         # docker container. Better approach is to set for destination and then
@@ -408,7 +424,7 @@ class SingularityContainer(Container, HasDockerLikeVolumes):
 
         working_directory = self.job_info.working_directory
         if not working_directory:
-            raise Exception("Cannot containerize command [%s] without defined working directory." % working_directory)
+            raise Exception(f"Cannot containerize command [{working_directory}] without defined working directory.")
 
         volumes_raw = self._expand_volume_str(self.destination_info.get("singularity_volumes", "$defaults"))
         preprocessed_volumes_list = preprocess_volumes(volumes_raw, self.container_type)
