@@ -42,7 +42,9 @@ class ContainerFinder:
 
     def __init__(self, app_info, mulled_resolution_cache=None):
         self.app_info = app_info
-        self.container_registry = ContainerRegistry(app_info, mulled_resolution_cache=mulled_resolution_cache)
+        self.mulled_resolution_cache = mulled_resolution_cache
+        self.default_container_registry = ContainerRegistry(app_info, mulled_resolution_cache=mulled_resolution_cache)
+        self.destination_container_registeries = {}
 
     def _enabled_container_types(self, destination_info):
         return [t for t in ALL_CONTAINER_TYPES if self.__container_type_enabled(t, destination_info)]
@@ -51,11 +53,26 @@ class ContainerFinder:
         """Regardless of destination properties - find best container for tool.
 
         Given container types and container.ToolInfo description of the tool."""
-        return self.container_registry.find_best_container_description(enabled_container_types, tool_info, **kwds)
+        return self.default_container_registry.find_best_container_description(enabled_container_types, tool_info, **kwds)
 
     def resolve(self, enabled_container_types, tool_info, **kwds):
         """Regardless of destination properties - find ResolvedContainerDescription for tool."""
-        return self.container_registry.resolve(enabled_container_types, tool_info, **kwds)
+        return self.default_container_registry.resolve(enabled_container_types, tool_info, **kwds)
+
+    def _container_registry_for_destination(self, destination_info):
+        destination_id = destination_info.get("id")  # Probably not the way to get the ID?
+        destination_container_registry = None
+        if destination_id and destination_id not in self.destination_container_registeries:
+            if 'container_resolvers' in destination_info:
+                destination_container_registry = ContainerRegistry(self.app_info, destination_info=destination_info, mulled_resolution_cache=self.mulled_resolution_cache)
+                self.destination_container_registeries[destination_id] = destination_container_registry
+        elif not destination_id and 'container_resolvers' in destination_info:
+            destination_container_registry = ContainerRegistry(self.app_info, destination_info=destination_info, mulled_resolution_cache=self.mulled_resolution_cache)
+
+        if destination_container_registry is None and destination_id and destination_id in self.destination_container_registeries:
+            destination_container_registry = self.destination_container_registeries[destination_id]
+
+        return destination_container_registry or self.default_container_registry
 
     def find_container(self, tool_info, destination_info, job_info):
         enabled_container_types = self._enabled_container_types(destination_info)
@@ -102,7 +119,8 @@ class ContainerFinder:
                     return container
 
         # Otherwise lets see if we can find container for the tool.
-        container_description = self.find_best_container_description(enabled_container_types, tool_info)
+        container_registry = self._container_registry_for_destination(destination_info)
+        container_description = container_registry.find_best_container_description(enabled_container_types, tool_info)
         container = __destination_container(container_description)
         if container:
             return container
@@ -124,44 +142,41 @@ class ContainerFinder:
         return NULL_CONTAINER
 
     def resolution_cache(self):
-        cache = ResolutionCache()
-        if self.container_registry.mulled_resolution_cache is not None:
-            cache.mulled_resolution_cache = self.container_registry.mulled_resolution_cache
-        return cache
+        return self.default_container_registry.get_resolution_cache()
 
     def __overridden_container_id(self, container_type, destination_info):
         if not self.__container_type_enabled(container_type, destination_info):
             return None
-        if "%s_container_id_override" % container_type in destination_info:
-            return destination_info.get("%s_container_id_override" % container_type)
-        if "%s_image_override" % container_type in destination_info:
+        if f"{container_type}_container_id_override" in destination_info:
+            return destination_info.get(f"{container_type}_container_id_override")
+        if f"{container_type}_image_override" in destination_info:
             return self.__build_container_id_from_parts(container_type, destination_info, mode="override")
 
     def __build_container_id_from_parts(self, container_type, destination_info, mode):
         repo = ""
         owner = ""
-        repo_key = "{}_repo_{}".format(container_type, mode)
-        owner_key = "{}_owner_{}".format(container_type, mode)
+        repo_key = f"{container_type}_repo_{mode}"
+        owner_key = f"{container_type}_owner_{mode}"
         if repo_key in destination_info:
-            repo = destination_info[repo_key] + "/"
+            repo = f"{destination_info[repo_key]}/"
         if owner_key in destination_info:
-            owner = destination_info[owner_key] + "/"
-        cont_id = repo + owner + destination_info["{}_image_{}".format(container_type, mode)]
-        tag_key = "{}_tag_{}".format(container_type, mode)
+            owner = f"{destination_info[owner_key]}/"
+        cont_id = repo + owner + destination_info[f"{container_type}_image_{mode}"]
+        tag_key = f"{container_type}_tag_{mode}"
         if tag_key in destination_info:
-            cont_id += ":" + destination_info[tag_key]
+            cont_id += f":{destination_info[tag_key]}"
         return cont_id
 
     def __default_container_id(self, container_type, destination_info):
         if not self.__container_type_enabled(container_type, destination_info):
             return None
-        key = "%s_default_container_id" % container_type
+        key = f"{container_type}_default_container_id"
         # Also allow docker_image...
         if key not in destination_info:
-            key = "%s_image" % container_type
+            key = f"{container_type}_image"
         if key in destination_info:
             return destination_info.get(key)
-        elif "%s_image_default" % container_type in destination_info:
+        elif f"{container_type}_image_default" in destination_info:
             return self.__build_container_id_from_parts(container_type, destination_info, mode="default")
         return None
 
@@ -177,7 +192,7 @@ class ContainerFinder:
         return CONTAINER_CLASSES[container_type](container_id, self.app_info, tool_info, destination_info, job_info, container_description)
 
     def __container_type_enabled(self, container_type, destination_info):
-        return asbool(destination_info.get("%s_enabled" % container_type, False))
+        return asbool(destination_info.get(f"{container_type}_enabled", False))
 
 
 class NullContainerFinder:
@@ -189,22 +204,36 @@ class NullContainerFinder:
 class ContainerRegistry:
     """Loop through enabled ContainerResolver plugins and find first match."""
 
-    def __init__(self, app_info, mulled_resolution_cache=None):
+    def __init__(self, app_info, destination_info=None, mulled_resolution_cache=None):
         self.resolver_classes = self.__resolvers_dict()
         self.enable_mulled_containers = app_info.enable_mulled_containers
         self.app_info = app_info
-        self.container_resolvers = self.__build_container_resolvers(app_info)
+        self.container_resolvers = self.__build_container_resolvers(app_info, destination_info)
         self.mulled_resolution_cache = mulled_resolution_cache
 
-    def __build_container_resolvers(self, app_info):
-        conf_file = getattr(app_info, 'containers_resolvers_config_file', None)
-        if not conf_file:
-            return self.__default_containers_resolvers()
-        if not os.path.exists(conf_file):
-            log.debug("Unable to find config file '%s'", conf_file)
-            return self.__default_containers_resolvers()
-        plugin_source = plugin_config.plugin_source_from_path(conf_file)
-        return self._parse_resolver_conf(plugin_source)
+    def __build_container_resolvers(self, app_info, destination_info):
+        app_conf_file = getattr(app_info, 'container_resolvers_config_file', None)
+        app_conf_dict = getattr(app_info, 'container_resolvers_config_dict', None)
+
+        if destination_info is not None:
+            conf_file = destination_info.get('container_resolvers_config_file', app_conf_file)
+            conf_dict = destination_info.get('container_resolvers', app_conf_dict)
+        else:
+            conf_file = app_conf_file
+            conf_dict = app_conf_dict
+
+        plugin_source = None
+        if conf_dict:
+            log.debug("Loading container resolution config inline from Galaxy or job configuration file")
+            plugin_source = plugin_config.plugin_source_from_dict(conf_dict)
+        elif conf_file and not os.path.exists(conf_file):
+            log.warning(f"Unable to find config file '{conf_file}'")
+        elif conf_file:
+            log.debug("Loading container resolution config from file '{conf_file}'")
+            plugin_source = plugin_config.plugin_source_from_path(conf_file)
+        if plugin_source:
+            return self._parse_resolver_conf(plugin_source)
+        return self.__default_container_resolvers()
 
     def _parse_resolver_conf(self, plugin_source):
         extra_kwds = {
@@ -212,7 +241,7 @@ class ContainerRegistry:
         }
         return plugin_config.load_plugins(self.resolver_classes, plugin_source, extra_kwds)
 
-    def __default_containers_resolvers(self):
+    def __default_container_resolvers(self):
         default_resolvers = [
             ExplicitContainerResolver(self.app_info),
             ExplicitSingularityContainerResolver(self.app_info),
@@ -225,15 +254,27 @@ class ContainerRegistry:
                 CachedMulledSingularityContainerResolver(self.app_info, namespace="local"),
                 MulledDockerContainerResolver(self.app_info, namespace="biocontainers"),
                 MulledSingularityContainerResolver(self.app_info, namespace="biocontainers"),
-                BuildMulledDockerContainerResolver(self.app_info),
-                BuildMulledSingularityContainerResolver(self.app_info),
             ])
+            # BuildMulledDockerContainerResolver and BuildMulledSingularityContainerResolver both need the docker daemon to build images.
+            # If docker is not available, we don't load them.
+            build_mulled_docker_container_resolver = BuildMulledDockerContainerResolver(self.app_info)
+            if build_mulled_docker_container_resolver.cli_available:
+                default_resolvers.extend([
+                    build_mulled_docker_container_resolver,
+                    BuildMulledSingularityContainerResolver(self.app_info),
+                ])
         return default_resolvers
 
     def __resolvers_dict(self):
         import galaxy.tool_util.deps.container_resolvers
         return plugin_config.plugins_dict(galaxy.tool_util.deps.container_resolvers, 'resolver_type')
 
+    def get_resolution_cache(self):
+        cache = ResolutionCache()
+        if self.mulled_resolution_cache is not None:
+            cache.mulled_resolution_cache = self.mulled_resolution_cache
+        return cache
+
     def find_best_container_description(self, enabled_container_types, tool_info, **kwds):
         """Yield best container description of supplied types matching tool info."""
         try:
@@ -243,7 +284,8 @@ class ContainerRegistry:
             return None
         return None if resolved_container_description is None else resolved_container_description.container_description
 
-    def resolve(self, enabled_container_types, tool_info, index=None, resolver_type=None, install=True, resolution_cache=None):
+    def resolve(self, enabled_container_types, tool_info, index=None, resolver_type=None, install=True, resolution_cache=None, session=None):
+        resolution_cache = resolution_cache or self.get_resolution_cache()
         for i, container_resolver in enumerate(self.container_resolvers):
             if index is not None and i != index:
                 continue
@@ -258,8 +300,8 @@ class ContainerRegistry:
             if not install and container_resolver.builds_on_resolution:
                 continue
 
-            container_description = container_resolver.resolve(enabled_container_types, tool_info, resolution_cache=resolution_cache)
-            log.info("Checking with container resolver [{}] found description [{}]".format(container_resolver, container_description))
+            container_description = container_resolver.resolve(enabled_container_types, tool_info, install=install, resolution_cache=resolution_cache, session=session)
+            log.info(f"Checking with container resolver [{container_resolver}] found description [{container_description}]")
             if container_description:
                 assert container_description.type in enabled_container_types
                 return ResolvedContainerDescription(container_resolver, container_description)
