@@ -22,6 +22,8 @@ import subprocess
 import sys
 import time
 
+import requests
+
 from ._cli import arg_parser
 from .mulled_build import (
     add_build_arguments,
@@ -40,11 +42,11 @@ def _fetch_repo_data(args):
     if not os.path.exists(repo_data):
         platform_tag = 'osx-64' if sys.platform == 'darwin' else 'linux-64'
         subprocess.check_call([
-            'wget', '--quiet', 'https://conda.anaconda.org/{}/{}/repodata.json.bz2'.format(channel, platform_tag),
-            '-O', '%s.bz2' % repo_data
+            'wget', '--quiet', f'https://conda.anaconda.org/{channel}/{platform_tag}/repodata.json.bz2',
+            '-O', f'{repo_data}.bz2'
         ])
         subprocess.check_call([
-            'bzip2', '-d', '%s.bz2' % repo_data
+            'bzip2', '-d', f'{repo_data}.bz2'
         ])
     return repo_data
 
@@ -58,6 +60,7 @@ def _new_versions(quay, conda):
 
 def run_channel(args, build_last_n_versions=1):
     """Build list of involucro commands (as shell snippet) to run."""
+    session = requests.session()
     for pkg_name, pkg_tests in get_affected_packages(args):
         repo_data = _fetch_repo_data(args)
         c = conda_versions(pkg_name, repo_data)
@@ -66,7 +69,7 @@ def run_channel(args, build_last_n_versions=1):
 
         if not args.force_rebuild:
             time.sleep(1)
-            q = quay_versions(args.namespace, pkg_name)
+            q = quay_versions(args.namespace, pkg_name, session)
             versions = _new_versions(q, c)
         else:
             versions = c
