@@ -5,18 +5,24 @@ import json
 import logging
 import sys
 import tempfile
+from datetime import (
+    datetime,
+    timezone,
+)
+
+import requests
 
 from .mulled_list import get_singularity_containers
-from .util import build_target, v2_image_name
+from .util import (
+    build_target,
+    MULLED_SOCKET_TIMEOUT,
+    v2_image_name,
+)
 
 try:
     from conda.cli.python_api import run_command
 except ImportError:
     run_command = None
-try:
-    import requests
-except ImportError:
-    requests = None
 
 try:
     from whoosh.fields import Schema
@@ -48,7 +54,7 @@ class QuaySearch():
 
         parameters = {'public': 'true', 'namespace': self.organization}
         r = requests.get(QUAY_API_URL, headers={
-                         'Accept-encoding': 'gzip'}, params=parameters, timeout=12)
+                         'Accept-encoding': 'gzip'}, params=parameters, timeout=MULLED_SOCKET_TIMEOUT)
         tmp_dir = tempfile.mkdtemp()
         schema = Schema(title=TEXT(stored=True), content=STORED)
         self.index = create_in(tmp_dir, schema)
@@ -79,7 +85,7 @@ class QuaySearch():
 
                 # get all repositories with suggested keywords
                 for suggestion in suggestions:
-                    search_string = "*%s*" % suggestion
+                    search_string = f"*{suggestion}*"
                     query = QueryParser(
                         "title", self.index.schema).parse(search_string)
                     results_tmp = searcher.search(query)
@@ -99,8 +105,8 @@ class QuaySearch():
         Function downloads additional information from quay.io to
         get the tag-field which includes the version number.
         """
-        url = "{}/{}/{}".format(QUAY_API_URL, self.organization, repository_string)
-        r = requests.get(url, headers={'Accept-encoding': 'gzip'})
+        url = f"{QUAY_API_URL}/{self.organization}/{repository_string}"
+        r = requests.get(url, headers={'Accept-encoding': 'gzip'}, timeout=MULLED_SOCKET_TIMEOUT)
 
         json_decoder = json.JSONDecoder()
         decoded_request = json_decoder.decode(r.text)
@@ -120,13 +126,15 @@ class CondaSearch():
         Function takes search_string variable and returns results from the bioconda channel in JSON format
 
         """
+        if run_command is None:
+            raise Exception(f"Invalid search destination. {deps_error_message('conda')}")
         raw_out, err, exit_code = run_command(
             'search', '-c',
             self.channel,
             search_string,
             use_exception_handler=True)
         if exit_code != 0:
-            logging.info('Search failed with: %s' % err)
+            logging.info(f'Search failed with: {err}')
             return []
         return [{'package': n.split()[0], 'version': n.split()[1], 'build': n.split()[2]} for n in raw_out.split('\n')[2:-1]]
 
@@ -136,34 +144,49 @@ class GitHubSearch():
     Tool to search the GitHub bioconda-recipes repo
     """
 
+    @staticmethod
+    def _check_response_rate_limit(response):
+        if response.status_code == 403 and "API rate limit exceeded" in response.json()["message"]:
+            # It can take tens of minutes before the rate limit window resets
+            message = "GitHub API rate limit exceeded."
+            rate_limit_reset_UTC_timestamp = response.headers.get("X-RateLimit-Reset")
+            if rate_limit_reset_UTC_timestamp:
+                rate_limit_reset_datetime = datetime.fromtimestamp(int(rate_limit_reset_UTC_timestamp), tz=timezone.utc)
+                message += f" The rate limit window will reset at {rate_limit_reset_datetime.isoformat()}."
+            raise Exception(message)
+
     def get_json(self, search_string):
         """
         Takes search_string variable and return results from the bioconda-recipes github repository in JSON format
+
+        DEPRECATED: this method is currently unreliable because the API query
+        sometimes succeeds but returns no items.
         """
         response = requests.get(
-            "https://api.github.com/search/code?q=%s+in:path+repo:bioconda/bioconda-recipes+path:recipes" % search_string).json()
-        return response
+            f"https://api.github.com/search/code?q={search_string}+in:path+repo:bioconda/bioconda-recipes+path:recipes",
+            timeout=MULLED_SOCKET_TIMEOUT,
+        )
+        self._check_response_rate_limit(response)
+        response.raise_for_status()
+        return response.json()
 
-    def process_json(self, json, search_string):
+    def process_json(self, json_response, search_string):
         """
         Take JSON input and process it, returning the required data
         """
-        json = json['items'][0:10]  # get top ten results
-
-        results = []
-
-        for result in json:
-            results.append({'name': result['name'], 'path': result['path']})
-        return results
+        top_10_items = json_response['items'][0:10]  # get top ten results
+        return [{'name': result['name'], 'path': result['path']} for result in top_10_items]
 
     def recipe_present(self, search_string):
         """
         Check if a recipe exists in bioconda-recipes which matches search_string exactly
         """
-        if requests.get("https://api.github.com/repos/bioconda/bioconda-recipes/contents/recipes/%s" % search_string).status_code == 200:
-            return True
-        else:
-            return False
+        response = requests.get(
+            f"https://api.github.com/repos/bioconda/bioconda-recipes/contents/recipes/{search_string}",
+            timeout=MULLED_SOCKET_TIMEOUT,
+        )
+        self._check_response_rate_limit(response)
+        return response.status_code == 200
 
 
 def get_package_hash(packages, versions):
@@ -184,7 +207,7 @@ def get_package_hash(packages, versions):
     if versions:
         hash_results['version_hash'] = package_hash.split(':')[1]
 
-    r = requests.get("https://quay.io/api/v1/repository/biocontainers/%s" % hash_results['package_hash'])
+    r = requests.get(f"https://quay.io/api/v1/repository/biocontainers/{hash_results['package_hash']}", timeout=MULLED_SOCKET_TIMEOUT)
     if r.status_code == 200:
         hash_results['container_present'] = True
         if versions:  # now test if the version hash is listed in the repository tags
@@ -219,27 +242,27 @@ def singularity_search(search_string):
 def readable_output(json, organization='biocontainers', channel='bioconda'):
 
     # if json is empty:
-    if sum([len(json[destination][results]) for destination in json for results in json[destination]]) == 0:
+    if sum(len(json[destination][results]) for destination in json for results in json[destination]) == 0:
         sys.stdout.write('No results found for that query.\n')
         return
 
     # return results for quay, conda and singularity together
-    if sum([len(json[destination][results]) for destination in ['quay', 'conda', 'singularity', ] for results in json.get(destination, [])]) > 0:
+    if sum(len(json[destination][results]) for destination in ['quay', 'conda', 'singularity', ] for results in json.get(destination, [])) > 0:
         sys.stdout.write("The query returned the following result(s).\n")
         # put quay, conda etc results as lists in lines
         lines = [['LOCATION', 'NAME', 'VERSION', 'COMMAND\n']]
-        for search_string, results in json.get('quay', {}).items():
+        for results in json.get('quay', {}).values():
             for result in results:
                 lines.append(['quay', result['package'], result['version'], 'docker pull quay.io/%s/%s:%s\n' %
                               (organization, result['package'], result['version'])])  # NOT a real solution
-        for search_string, results in json.get('conda', {}).items():
+        for results in json.get('conda', {}).values():
             for result in results:
-                lines.append(['conda', result['package'], '{}--{}'.format(result['version'], result['build']),
-                              'conda install -c {} {}={}={}\n'.format(channel, result['package'], result['version'], result['build'])])
-        for search_string, results in json.get('singularity', {}).items():
+                lines.append(['conda', result['package'], f"{result['version']}--{result['build']}",
+                              f"conda install -c {channel} {result['package']}={result['version']}={result['build']}\n"])
+        for results in json.get('singularity', {}).values():
             for result in results:
                 lines.append(['singularity', result['package'], result['version'],
-                              'wget https://depot.galaxyproject.org/singularity/{}:{}\n'.format(result['package'], result['version'])])
+                              f"wget https://depot.galaxyproject.org/singularity/{result['package']}:{result['version']}\n"])
 
         col_width0, col_width1, col_width2 = (max(len(
             line[n]) for line in lines) + 2 for n in (0, 1, 2))  # def max col widths for the output
@@ -256,7 +279,7 @@ def readable_output(json, organization='biocontainers', channel='bioconda'):
         lines = [['QUERY', 'LOCATION\n']]
         for recipe in json['github_recipe_present']['recipes']:
             lines.append(
-                [recipe, "https://api.github.com/repos/bioconda/bioconda-recipes/contents/recipes/%s\n" % recipe])
+                [recipe, f"https://api.github.com/repos/bioconda/bioconda-recipes/contents/recipes/{recipe}\n"])
 
         col_width0 = max(len(line[0]) for line in lines) + 2
 
@@ -264,7 +287,7 @@ def readable_output(json, organization='biocontainers', channel='bioconda'):
             sys.stdout.write(
                 "".join((line[0].ljust(col_width0), line[1])))  # output
 
-    if sum([len(json['github'][results]) for results in json.get('github', [])]) > 0:
+    if sum(len(json['github'][results]) for results in json.get('github', [])) > 0:
         sys.stdout.write('\n' if 'lines' in locals() else '')
         sys.stdout.write(
             "Other result(s) on the bioconda-recipes GitHub repository:\n")
@@ -272,7 +295,7 @@ def readable_output(json, organization='biocontainers', channel='bioconda'):
         for search_string, results in json.get('github', {}).items():
             for result in results:
                 lines.append([search_string, result['name'],
-                              'https://github.com/bioconda/bioconda-recipes/tree/master/%s\n' % result['path']])
+                              f"https://github.com/bioconda/bioconda-recipes/tree/master/{result['path']}\n"])
 
         # def max col widths for the output
         col_width0, col_width1 = (
@@ -283,15 +306,22 @@ def readable_output(json, organization='biocontainers', channel='bioconda'):
                 (line[0].ljust(col_width0), line[1].ljust(col_width1), line[2])))  # output
 
 
+def deps_error_message(package):
+    return f"Required dependency [{package}] is not installed. Run 'pip install galaxy-tool-util[mulled]'."
+
+
 def main(argv=None):
     if Schema is None:
-        sys.stdout.write(
-            "Required dependencies are not installed. Run 'pip install Whoosh'.\n")
+        sys.stdout.write(deps_error_message("Whoosh"))
         return
 
+    destination_defaults = ['quay', 'singularity', 'github']
+    if run_command is not None:
+        destination_defaults.append('conda')
+
     parser = argparse.ArgumentParser(
         description='Searches in a given quay organization for a repository')
-    parser.add_argument('-d', '--destination', dest='search_dest', nargs='+', default=['quay', 'conda', 'singularity'],
+    parser.add_argument('-d', '--destination', dest='search_dest', nargs='+', default=destination_defaults,
                         help="Choose where to search. Options are 'conda', 'quay', 'singularity' and 'github'. If no option are given, all will be searched.")
     parser.add_argument('-o', '--organization', dest='organization_string', default="biocontainers",
                         help='Change quay organization to search; default is biocontainers.')
@@ -327,10 +357,11 @@ def main(argv=None):
         github = GitHubSearch()
 
         for item in args.search:
-            github_json = github.get_json(item)
-            github_results[item] = github.process_json(github_json, item)
             if github.recipe_present(item):
                 github_recipe_present.append(item)
+            else:
+                github_json = github.get_json(item)
+                github_results[item] = github.process_json(github_json, item)
 
         json_results['github'] = github_results
         json_results['github_recipe_present'] = {
