@@ -6,6 +6,7 @@ from abc import (
     abstractmethod,
     abstractproperty,
 )
+from typing import Any, Dict
 
 import yaml
 
@@ -18,7 +19,7 @@ class DependencyResolver(Dictifiable, metaclass=ABCMeta):
     """Abstract description of a technique for resolving container images for tool execution."""
 
     # Keys for dictification.
-    dict_collection_visible_keys = ['resolver_type', 'resolves_simple_dependencies', 'can_uninstall_dependencies']
+    dict_collection_visible_keys = ['resolver_type', 'resolves_simple_dependencies', 'can_uninstall_dependencies', 'read_only']
     # A "simple" dependency is one that does not depend on the the tool
     # resolving the dependency. Classic tool shed dependencies are non-simple
     # because the repository install context is used in dependency resolution
@@ -26,8 +27,8 @@ class DependencyResolver(Dictifiable, metaclass=ABCMeta):
     # resolution.
     disabled = False
     resolves_simple_dependencies = True
-    can_uninstall_dependencies = False
-    config_options = {}
+    config_options: Dict[str, Any] = {}
+    read_only = True
 
     @abstractmethod
     def resolve(self, requirement, **kwds):
@@ -42,6 +43,22 @@ class DependencyResolver(Dictifiable, metaclass=ABCMeta):
         request version is 'default'.)
         """
 
+    def install_dependency(self, name, version, type, **kwds):
+        if self.read_only:
+            return False
+        else:
+            return self._install_dependency(name, version, type, **kwds)
+
+    def _install_dependency(self, name, version, type, **kwds):
+        """ Attempt to install this dependency if a recipe to do so
+        has been registered in some way.
+        """
+        return False
+
+    @property
+    def can_uninstall_dependencies(self):
+        return not self.read_only
+
 
 class MultipleDependencyResolver:
     """Variant of DependencyResolver that can optionally resolve multiple dependencies together."""
@@ -236,18 +253,6 @@ class SpecificationPatternDependencyResolver(SpecificationAwareDependencyResolve
         return requirement
 
 
-class InstallableDependencyResolver(metaclass=ABCMeta):
-    """ Mix this into a ``DependencyResolver`` and implement to indicate
-    the dependency resolver can attempt to install new dependencies.
-    """
-
-    @abstractmethod
-    def install_dependency(self, name, version, type, **kwds):
-        """ Attempt to install this dependency if a recipe to do so
-        has been registered in some way.
-        """
-
-
 class Dependency(Dictifiable, metaclass=ABCMeta):
     dict_collection_visible_keys = ['dependency_type', 'exact', 'name', 'version', 'cacheable']
     cacheable = False
@@ -269,7 +274,7 @@ class Dependency(Dictifiable, metaclass=ABCMeta):
         """
         Return a message describing this dependency
         """
-        return "Using dependency {} version {} of type {}".format(self.name, self.version, self.dependency_type)
+        return f"Using dependency {self.name} version {self.version} of type {self.dependency_type}"
 
 
 class ContainerDependency(Dependency):
@@ -314,7 +319,7 @@ class NullDependency(Dependency):
         """
         Return a message describing this dependency
         """
-        return "Dependency %s not found." % self.name
+        return f"Dependency {self.name} not found."
 
     def shell_commands(self):
         return None
