@@ -13,7 +13,6 @@ from . import (
     Dependency,
     DependencyException,
     DependencyResolver,
-    InstallableDependencyResolver,
     ListableDependencyResolver,
     MappableDependencyResolver,
     MultipleDependencyResolver,
@@ -39,9 +38,8 @@ from ..conda_util import (
 DEFAULT_BASE_PATH_DIRECTORY = "_conda"
 DEFAULT_CONDARC_OVERRIDE = "_condarc"
 # Conda channel order from highest to lowest, following the one used in
-# https://github.com/bioconda/bioconda-recipes/blob/master/config.yml , but
-# adding `iuc` as first channel (for Galaxy-specific packages)
-DEFAULT_ENSURE_CHANNELS = "iuc,conda-forge,bioconda,defaults"
+# https://github.com/bioconda/bioconda-recipes/blob/master/config.yml
+DEFAULT_ENSURE_CHANNELS = "conda-forge,bioconda,defaults"
 CONDA_SOURCE_CMD = """[ "$(basename "$CONDA_DEFAULT_ENV")" = "$(basename '{environment_path}')" ] || {{
 MAX_TRIES=3
 COUNT=0
@@ -65,7 +63,7 @@ done
 log = logging.getLogger(__name__)
 
 
-class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, ListableDependencyResolver, InstallableDependencyResolver, SpecificationPatternDependencyResolver, MappableDependencyResolver):
+class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, ListableDependencyResolver, SpecificationPatternDependencyResolver, MappableDependencyResolver):
     dict_collection_visible_keys = DependencyResolver.dict_collection_visible_keys + ['prefix', 'versionless', 'ensure_channels', 'auto_install', 'auto_init', 'use_local']
     resolver_type = "conda"
     config_options = {
@@ -81,7 +79,8 @@ class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, Li
     _specification_pattern = re.compile(r"https\:\/\/anaconda.org\/\w+\/\w+")
 
     def __init__(self, dependency_manager, **kwds):
-        self.can_uninstall_dependencies = True
+        read_only = _string_as_bool(kwds.get('read_only', 'false'))
+        self.read_only = read_only
         self._setup_mapping(dependency_manager, **kwds)
         self.versionless = _string_as_bool(kwds.get('versionless', 'false'))
         self.dependency_manager = dependency_manager
@@ -161,13 +160,16 @@ class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, Li
         final_return_code = 0
         for env, return_code in zip(environments, return_codes):
             if return_code == 0:
-                log.debug("Conda environment '%s' successfully removed." % env)
+                log.debug(f"Conda environment '{env}' successfully removed.")
             else:
-                log.debug("Conda environment '%s' could not be removed." % env)
+                log.debug(f"Conda environment '{env}' could not be removed.")
                 final_return_code = return_code
         return final_return_code
 
     def install_all(self, conda_targets):
+        if self.read_only:
+            return False
+
         env = self.merged_environment_name(conda_targets)
         return_code = install_conda_targets(conda_targets, conda_context=self.conda_context, env_name=env)
         if return_code != 0:
@@ -177,7 +179,7 @@ class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, Li
             is_installed = self.conda_context.has_env(env)
 
         if not is_installed:
-            log.debug("Removing failed conda install of {}".format(str(conda_targets)))
+            log.debug(f"Removing failed conda install of {str(conda_targets)}")
             cleanup_failed_install_of_environment(env, conda_context=self.conda_context)
 
         return is_installed
@@ -249,7 +251,7 @@ class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, Li
     def merged_environment_name(self, conda_targets):
         if len(conda_targets) > 1:
             # For continuity with mulled containers this is kind of nice.
-            return "mulled-v1-%s" % hash_conda_packages(conda_targets)
+            return f"mulled-v1-{hash_conda_packages(conda_targets)}"
         else:
             assert len(conda_targets) == 1
             return conda_targets[0].install_environment
@@ -336,10 +338,13 @@ class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, Li
             version = install_target.version
             yield self._to_requirement(name, version)
 
-    def install_dependency(self, name, version, type, **kwds):
+    def _install_dependency(self, name, version, type, **kwds):
         "Returns True on (seemingly) successfull installation"
+        # should be checked before called
+        assert not self.read_only
+
         if type != "package":
-            log.warning("Cannot install dependencies of type '%s'" % type)
+            log.warning(f"Cannot install dependencies of type '{type}'")
             return False
 
         if self.versionless:
@@ -363,7 +368,7 @@ class CondaDependencyResolver(DependencyResolver, MultipleDependencyResolver, Li
                 conda_target, conda_context=self.conda_context
             )
         if not is_installed:
-            log.debug("Removing failed conda install of {}, version '{}'".format(name, version))
+            log.debug(f"Removing failed conda install of {name}, version '{version}'")
             cleanup_failed_install(conda_target, conda_context=self.conda_context)
 
         return is_installed
@@ -405,9 +410,7 @@ class MergedCondaDependency(Dependency):
             # On explicit testing the only such requirement I am aware of is samtools - and it seems to work
             # fine with just appending the PATH as done below. Other tools may require additional
             # variables in the future.
-            return """export PATH=$PATH:'{}/bin' """.format(
-                self.environment_path,
-            )
+            return f"""export PATH=$PATH:'{self.environment_path}/bin' """
         else:
             return CONDA_SOURCE_CMD.format(
                 activate_path=self.activate,
@@ -474,9 +477,7 @@ class CondaDependency(Dependency):
             # On explicit testing the only such requirement I am aware of is samtools - and it seems to work
             # fine with just appending the PATH as done below. Other tools may require additional
             # variables in the future.
-            return """export PATH=$PATH:'{}/bin' """.format(
-                self.environment_path,
-            )
+            return f"""export PATH=$PATH:'{self.environment_path}/bin' """
         else:
             return CONDA_SOURCE_CMD.format(
                 activate_path=self.activate,
